@@ -1,16 +1,30 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
+#
+# Copyright 2022 Greg Albrecht <oss@undef.net>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# Author:: Greg Albrecht W2GMD <oss@undef.net>
+#
 
-"""AIS Cursor-On-Target Gateway Functions."""
+"""AISCOT Functions."""
 
 import csv
-import datetime
-import platform
-
+import configparser
 import xml.etree.ElementTree as ET
 
-import pytak
+from typing import Union
 
+import pytak
 import aiscot
 
 __author__ = "Greg Albrecht W2GMD <oss@undef.net>"
@@ -18,150 +32,192 @@ __copyright__ = "Copyright 2022 Greg Albrecht"
 __license__ = "Apache License, Version 2.0"
 
 
+def create_tasks(
+    config: Union[dict, configparser.ConfigParser], clitool: pytak.CLITool
+) -> set:
+    """
+    Creates specific coroutine task set for this application.
+
+    Parameters
+    ----------
+    config : `dict`, `configparser.ConfigParser`
+        A `dict` of configuration parameters & values.
+    clitool : `pytak.CLITool`
+        A PyTAK Worker class instance.
+
+    Returns
+    -------
+    `set`
+        Set of PyTAK Worker classes for this application.
+    """
+    return set([aiscot.AISWorker(clitool.tx_queue, config)])
+
+
 # pylint: disable=too-many-locals, too-many-branches, too-many-statements
 def ais_to_cot_xml(
-    msg: dict, stale: int = None, cot_type: str = "", known_craft: dict = None
+    craft: dict, config: Union[dict, None] = None, known_craft: Union[dict, None] = None
 ) -> ET.Element:
     """
     Converts an AIS Sentence to a Cursor-On-Target Event.
 
-    :param msg: AIS Sentence to convert to COT.
-    :type msg: `dict`
-    :param cot_stale: Number of Seconds from now to mark the COT Event stale.
-    :type cot_stale: `int`
+    Parameters
+    ----------
+    msg : `dict`
+        AIS sentence serialized as a `dict`.
+    clitool : `pytak.CLITool`
+        A PyTAK Worker class instance.
+
+    Returns
+    -------
+    `xml.etree.ElementTree.Element`
+        Cursor-On-Target XML ElementTree object.
     """
+    lat = craft.get("lat", craft.get("LATITUDE"))
+    lon = craft.get("lon", craft.get("LONGITUDE"))
+    mmsi = craft.get("mmsi", craft.get("MMSI"))
+
+    if lat is None or lon is None or mmsi is None:
+        return None
+
     known_craft = known_craft or {}
-    time = datetime.datetime.now(datetime.timezone.utc)
+    remarks_fields = []
+    config: dict = config or {}
 
-    cot_stale = stale or known_craft.get("STALE") or aiscot.DEFAULT_COT_STALE
+    cot_stale = (
+        config.get("COT_STALE", None)
+        or known_craft.get("STALE")
+        or aiscot.DEFAULT_COT_STALE
+    )
+    cot_type = (
+        config.get("COT_TYPE", None)
+        or known_craft.get("COT")
+        or aiscot.DEFAULT_COT_TYPE
+    )
+    cot_host_id = config.get("COT_HOST_ID")
 
-    mmsi = msg.get("mmsi", msg.get("MMSI"))
+    aiscotx = ET.Element("_aiscot_")
+    aiscotx.set("cot_host_id", cot_host_id)
 
-    ais_name: str = msg.get("name", msg.get("NAME", "")).replace("@", "").strip()
-    shipname: str = msg.get("shipname", aiscot.get_shipname(mmsi))
+    ais_name: str = craft.get("name", craft.get("NAME", "")).replace("@", "").strip()
+    shipname: str = craft.get("shipname", aiscot.get_shipname(mmsi))
+    vessel_type: str = craft.get("type", craft.get("TYPE"))
+
+    if ais_name:
+        remarks_fields.append(f"AIS Name: {ais_name}")
+        aiscotx.set("ais_name", ais_name)
 
     if shipname:
         ais_name = shipname
+        remarks_fields.append(f"Shipname: {shipname}")
+        aiscotx.set("shipname", shipname)
+
+    uid = f"MMSI-{mmsi}"
 
-    name = f"MMSI-{mmsi}"
     _name = known_craft.get("NAME") or ais_name
     if _name:
         callsign = _name
     else:
         callsign = mmsi
 
-    if known_craft.get("NAME"):
-        print(f"NAME = {_name} MMSI = {mmsi}")
-
-    cot_type = known_craft.get("COT") or cot_type or aiscot.DEFAULT_COT_TYPE
-
     country: str = aiscot.get_mid(mmsi)
-    if country and "United States of America" in country:
-        cot_type = "a-f" + cot_type[3:]
-    elif country:
+    if country:
         cot_type = "a-n" + cot_type[3:]
+        remarks_fields.append(f"Country: {country}")
+        aiscotx.set("country", country)
+        if "United States of America" in country:
+            cot_type = "a-f" + cot_type[3:]
+
+    if vessel_type:
+        ais_name = shipname
+        remarks_fields.append(f"Type: {vessel_type}")
+        aiscotx.set("type", str(vessel_type))
+
+    if mmsi:
+        remarks_fields.append(f"MMSI: {mmsi}")
+        aiscotx.set("mmsi", str(mmsi))
 
     aton: bool = aiscot.get_aton(mmsi)
+    aiscotx.set("aton", str(aton))
     if aton:
         cot_type = "a-n-S-N"
         cot_stale = 86400  # 1 Day
         callsign = f"AtoN {callsign}"
+        remarks_fields.append(f"AtoN: {aton}")
 
     uscg: bool = aiscot.get_sar(mmsi)
+    aiscotx.set("uscg", str(uscg))
     if uscg:
         cot_type = "a-f-S-X-L"
+        remarks_fields.append(f"USCG: {uscg}")
 
     crs: bool = aiscot.get_crs(mmsi)
+    aiscotx.set("crs", str(crs))
     if crs:
         cot_type = "a-f-G-I-U-T"
         cot_stale = 86400  # 1 Day
         callsign = f"USCG CRS {callsign}"
+        remarks_fields.append(f"USCG CRS: {crs}")
 
+    # Point
     point = ET.Element("point")
-    point.set("lat", str(msg.get("lat", msg.get("LATITUDE"))))
-    point.set("lon", str(msg.get("lon", msg.get("LONGITUDE"))))
+    point.set("lat", str(lat))
+    point.set("lon", str(lon))
     point.set("hae", "9999999.0")
     point.set("le", "9999999.0")
     point.set("ce", "9999999.0")
 
+    # Track
+    track = ET.Element("track")
+    heading: float = craft.get("heading", craft.get("HEADING", 0))  # * 0.1
+    track.set("course", str(heading))
+
+    # AIS Speed over ground: 0.1-knot (0.19 km/h) resolution from
+    #                    0 to 102 knots (189 km/h)
+    # COT Speed is meters/second
+    sog: float = craft.get("speed", craft.get("SPEED", 0)) * 0.1 / 1.944
+    if sog != 0.0:
+        speed = str(sog)
+    else:
+        speed = "9999999.0"
+    track.set("speed", speed)
+
+    # Contact
     contact = ET.Element("contact")
     contact.set("callsign", str(callsign))
 
-    track = ET.Element("track")
-    track.set("course", str(msg.get("heading", msg.get("HEADING", 0))))
-
-    # Speed over ground: 0.1-knot (0.19 km/h) resolution from
-    #                    0 to 102 knots (189 km/h)
-    sog = int(msg.get("speed", msg.get("SPEED", 0)))
-    if sog:
-        track.set("speed", str(sog * 0.514444))
-    else:
-        track.set("speed", "9999999.0")
+    remarks = ET.Element("remarks")
+    remarks_fields.append(f"{cot_host_id}")
+    _remarks = " ".join(list(filter(None, remarks_fields)))
+    remarks.text = _remarks
 
     detail = ET.Element("detail")
-    detail.set("uid", name)
-    detail.append(contact)
+    detail.set("uid", uid)
     detail.append(track)
-
-    remarks = ET.Element("remarks")
-
-    if crs:
-        remark = f"USCG CRS {mmsi}"
-        if ais_name:
-            remark = f"USCG CRS {ais_name} {mmsi}"
-    else:
-        _remark = f"AtoN/{aton} MMSI/{mmsi} Co/{country}"
-        if ais_name:
-            remark = f"{ais_name} {_remark}"
-        else:
-            remark = _remark
-
-    _remarks = f"{remark} Ty/{msg.get('type', msg.get('TYPE'))} Sn/{shipname}"
-    remarks.text = f"{_remarks} aiscot@{platform.node()}"
+    detail.append(contact)
     detail.append(remarks)
 
     root = ET.Element("event")
-
     root.set("version", "2.0")
     root.set("type", cot_type)
-    root.set("uid", name)
+    root.set("uid", uid)
     root.set("how", "m-g")
-    root.set("time", time.strftime(pytak.ISO_8601_UTC))
-    root.set("start", time.strftime(pytak.ISO_8601_UTC))
-    root.set(
-        "stale",
-        (time + datetime.timedelta(seconds=int(cot_stale))).strftime(
-            pytak.ISO_8601_UTC
-        ),
-    )
+    root.set("time", pytak.cot_time())
+    root.set("start", pytak.cot_time())
+    root.set("stale", pytak.cot_time(cot_stale))
 
     root.append(point)
     root.append(detail)
+    root.append(aiscotx)
 
     return root
 
 
 def ais_to_cot(
-    ais_msg: dict, stale: int = None, cot_type: str = "", known_craft: dict = None
-) -> str:
-    """
-    Converts an AIS Sentence to a Cursor-on-Target Event as a Python `str` String.
-
-    See `ais_to_cot_xml()` for XML Object output version of this function.
-
-    :param ais_msg: AIS Sentence to convert to CoT.
-    :type ais_msg: `dict`
-    :param cot_stale: Number of Seconds from now to mark the CoT Event stale.
-    :type cot_stale: `int`
-    """
-    lat = ais_msg.get("lat")
-    lon = ais_msg.get("lon")
-    mmsi = ais_msg.get("mmsi")
-
-    if lat is None or lon is None or mmsi is None:
-        return None
-
-    return ET.tostring(ais_to_cot_xml(ais_msg, stale, cot_type, known_craft))
+    craft: dict, config: Union[dict, None] = None, known_craft: Union[dict, None] = None
+) -> bytes:
+    """Wrapper for `ais_to_cot_xml` that returns COT as an XML string."""
+    cot: ET.ElementTree = ais_to_cot_xml(craft, config, known_craft)
+    return ET.tostring(cot) if cot else None
 
 
 def _read_known_craft_fd(csv_fd) -> list:
@@ -169,12 +225,12 @@ def _read_known_craft_fd(csv_fd) -> list:
 
     Parameters
     ----------
-    csv_fd : `fd`
+    csv_fd : fd
         FD Object.
 
     Returns
     -------
-    `list`
+    list
         Rows of `dict`s of CSV FD.
     """
     all_rows: list = []
@@ -203,7 +259,7 @@ def read_known_craft(csv_file: str) -> list:
     return all_rows
 
 
-def read_mid_db_file(csv_file: str = aiscot.MID_DB_FILE) -> list:
+def read_mid_db_file(csv_file: str = aiscot.DEFAULT_MID_DB_FILE) -> list:
     """Reads the MID_DB_FILE file into a `dict`"""
     mid_digits: list = []
     mid_allocated_to: list = []
@@ -217,7 +273,7 @@ def read_mid_db_file(csv_file: str = aiscot.MID_DB_FILE) -> list:
     return dict(zip(mid_digits, mid_allocated_to))
 
 
-def read_ship_db_file(csv_file: str = aiscot.SHIP_DB_FILE) -> list:
+def read_ship_db_file(csv_file: str = aiscot.DEFAULT_SHIP_DB_FILE) -> list:
     """Reads the SHIP_DB_FILE file into a `list`"""
     all_rows: list = []
     fields: list = ["MMSI", "name", "unk", "vtype"]
